<?php
namespace PortalPeople\invoicing\services;

use Yii;

class InvoiceGenerationServiceB2C
{
    /**
     * Generate and send invoice for B2C
     * @param int $cartId
     * @param string $email
     * @return mixed
     */
    public static function generateAndSendInvoice($cartId, $email)
    {
        // TODO: Move/refactor B2C invoice logic here
        // Example: create invoice, attach PDF, send to $email
        // Use B2C-specific business rules
        return true;
    }
}
